@@ -1,6 +1,6 @@
 /* Basic movement functions
 
-   Copyright (c) 1997-2004, 2008-2009, 2011 Free Software Foundation, Inc.
+   Copyright (c) 1997-2014 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -53,7 +53,7 @@ END_DEFUN
  * Get the goal column.  Take care of expanding tabulations.
  */
 size_t
-get_goalc_bp (Buffer * bp, size_t o)
+get_goalc_bp (Buffer bp, size_t o)
 {
   size_t col = 0, t = tab_width (bp);
   size_t start = buffer_start_of_line (bp, o), end = o - start;
@@ -121,7 +121,7 @@ Beginning of buffer is position 1.
   if (ok == leNIL || n >= LONG_MAX - 1)
     return leNIL;
 
-  goto_offset (MIN (get_buffer_size (cur_bp), MAX (n, 1) - 1));
+  goto_offset (MIN (get_buffer_size (cur_bp), (size_t)MAX (n, 1) - 1));
 }
 END_DEFUN
 
@@ -210,7 +210,7 @@ DEFUN ("scroll-down", scroll_down)
 Scroll text of current window downward near full screen.
 +*/
 {
-  ok = execute_with_uniarg (false, uniarg, scroll_down, scroll_up);
+  ok = execute_with_uniarg (uniarg, scroll_down, scroll_up);
 }
 END_DEFUN
 
@@ -219,6 +219,6 @@ DEFUN ("scroll-up", scroll_up)
 Scroll text of current window upward near full screen.
 +*/
 {
-  ok = execute_with_uniarg (false, uniarg, scroll_up, scroll_down);
+  ok = execute_with_uniarg (uniarg, scroll_up, scroll_down);
 }
 END_DEFUN
