@@ -1,6 +1,6 @@
 /* Key bindings and extended commands
 
-   Copyright (c) 1997-2011 Free Software Foundation, Inc.
+   Copyright (c) 1997-2012 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -211,9 +211,8 @@ self_insert_command (void)
   deactivate_mark ();
   if (key <= 0xff)
     {
-      if (isspace (key) && get_buffer_autofill (cur_bp) &&
-          get_goalc () > (size_t) get_variable_number ("fill-column"))
-        fill_break_line ();
+      if (isspace (key) && get_buffer_autofill (cur_bp))
+        ret = fill_break_line () != -1;
       insert_char (key);
     }
   else
@@ -231,7 +230,7 @@ Insert the character you type.
 Whichever character you type to run this command is inserted.
 +*/
 {
-  ok = execute_with_uniarg (true, uniarg, self_insert_command, NULL);
+  ok = execute_with_uniarg (uniarg, self_insert_command, NULL);
 }
 END_DEFUN
 
@@ -254,6 +253,7 @@ le *
 call_command (Function f, int uniarg, bool uniflag, le *branch)
 {
   thisflag = lastflag & FLAG_DEFINING_MACRO;
+  undo_start_sequence ();
 
   /* Reset last_uniarg before function call, so recursion (e.g. in
      macros) works. */
@@ -270,9 +270,7 @@ call_command (Function f, int uniarg, bool uniflag, le *branch)
   if (lastflag & FLAG_DEFINING_MACRO && thisflag & FLAG_DEFINING_MACRO)
     add_cmd_to_macro ();
 
-  if (cur_bp && last_command () != F_undo)
-    set_buffer_next_undop (cur_bp, get_buffer_last_undop (cur_bp));
-
+  undo_end_sequence ();
   lastflag = thisflag;
 
   return ok;
@@ -491,7 +489,7 @@ walk_bindings_tree (Binding tree, gl_list_t keys,
           astr key = astr_new ();
           for (size_t j = 0; j < gl_list_size (keys); j++)
             {
-              astr_cat (key, (castr) gl_list_get_at (keys, j));
+              astr_cat (key, (const_astr) gl_list_get_at (keys, j));
               astr_cat_char (key, ' ');
             }
           astr_cat (key, chordtodesc (p->key));
@@ -539,7 +537,7 @@ Print message listing key sequences that invoke the command DEFINITION.
 Argument is a command name.
 +*/
 {
-  castr name = minibuf_read_function_name ("Where is command: ");
+  const_astr name = minibuf_read_function_name ("Where is command: ");
   gather_bindings_state g;
 
   ok = leNIL;
