@@ -1,6 +1,6 @@
 /* Buffer-oriented functions
 
-   Copyright (c) 1997-2006, 2008-2012 Free Software Foundation, Inc.
+   Copyright (c) 1997-2014 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -62,48 +62,49 @@ struct Buffer
 /* Buffer methods that know about the gap. */
 
 void
-set_buffer_text (Buffer *bp, estr es)
+set_buffer_text (Buffer bp, estr es)
 {
   bp->text = es;
 }
 
-castr
-get_buffer_pre_point (Buffer *bp)
+const_astr
+get_buffer_pre_point (Buffer bp)
 {
-  return castr_new_nstr (astr_cstr (bp->text.as), get_buffer_pt (bp));
+  return const_astr_new_nstr (astr_cstr (estr_get_as (bp->text)), bp->pt);
 }
 
-castr
-get_buffer_post_point (Buffer *bp)
+const_astr
+get_buffer_post_point (Buffer bp)
 {
-  return castr_new_nstr (astr_cstr (bp->text.as) + bp->pt + bp->gap,
-                         astr_len (bp->text.as) - (bp->pt + bp->gap));
+  size_t post_gap = bp->pt + bp->gap;
+  const_astr as = estr_get_as (bp->text);
+  return const_astr_new_nstr (astr_cstr (as) + post_gap, astr_len (as) - post_gap);
 }
 
 size_t
-get_buffer_pt (Buffer *bp)
+get_buffer_pt (Buffer bp)
 {
   return bp->pt;
 }
 
 static void
-set_buffer_pt (Buffer *bp, size_t o)
+set_buffer_pt (Buffer bp, size_t o)
 {
   if (o < bp->pt)
     {
-      astr_move (bp->text.as, o + bp->gap, o, bp->pt - o);
-      astr_set (bp->text.as, o, '\0', MIN (bp->pt - o, bp->gap));
+      astr_move (estr_get_as (bp->text), o + bp->gap, o, bp->pt - o);
+      astr_set (estr_get_as (bp->text), o, '\0', MIN (bp->pt - o, bp->gap));
     }
   else if (o > bp->pt)
     {
-      astr_move (bp->text.as, bp->pt, bp->pt + bp->gap, o - bp->pt);
-      astr_set (bp->text.as, o + bp->gap - MIN (o - bp->pt, bp->gap), '\0', MIN (o - bp->pt, bp->gap));
+      astr_move (estr_get_as (bp->text), bp->pt, bp->pt + bp->gap, o - bp->pt);
+      astr_set (estr_get_as (bp->text), o + bp->gap - MIN (o - bp->pt, bp->gap), '\0', MIN (o - bp->pt, bp->gap));
     }
   bp->pt = o;
 }
 
 static inline size_t
-realo_to_o (Buffer *bp, size_t o)
+realo_to_o (Buffer bp, size_t o)
 {
   if (o == SIZE_MAX)
     return o;
@@ -114,19 +115,19 @@ realo_to_o (Buffer *bp, size_t o)
 }
 
 static inline size_t
-o_to_realo (Buffer *bp, size_t o)
+o_to_realo (Buffer bp, size_t o)
 {
   return o < bp->pt ? o : o + bp->gap;
 }
 
 size_t
-get_buffer_size (Buffer * bp)
+get_buffer_size (Buffer bp)
 {
-  return realo_to_o (bp, astr_len (bp->text.as));
+  return realo_to_o (bp, astr_len (estr_get_as (bp->text)));
 }
 
 size_t
-buffer_line_len (Buffer *bp, size_t o)
+buffer_line_len (Buffer bp, size_t o)
 {
   return realo_to_o (bp, estr_end_of_line (bp->text, o_to_realo (bp, o))) -
     realo_to_o (bp, estr_start_of_line (bp->text, o_to_realo (bp, o)));
@@ -138,7 +139,7 @@ buffer_line_len (Buffer *bp, size_t o)
 #define MIN_GAP 1024 /* Minimum gap size after resize. */
 #define MAX_GAP 4096 /* Maximum permitted gap size. */
 bool
-replace_estr (size_t del, estr es)
+replace_estr (size_t del, const_estr es)
 {
   if (warn_if_readonly_buffer ())
     return false;
@@ -151,12 +152,12 @@ replace_estr (size_t del, estr es)
   size_t added_gap = oldgap + del < newlen ? MIN_GAP : 0;
   if (added_gap > 0)
     { /* If gap would vanish, open it to MIN_GAP. */
-      astr_insert (cur_bp->text.as, cur_bp->pt, (newlen + MIN_GAP) - (oldgap + del));
+      astr_insert (estr_get_as (cur_bp->text), cur_bp->pt, (newlen + MIN_GAP) - (oldgap + del));
       cur_bp->gap = MIN_GAP;
     }
   else if (oldgap + del > MAX_GAP + newlen)
     { /* If gap would be larger than MAX_GAP, restrict it to MAX_GAP. */
-      astr_remove (cur_bp->text.as, cur_bp->pt + newlen + MAX_GAP, (oldgap + del) - (MAX_GAP + newlen));
+      astr_remove (estr_get_as (cur_bp->text), cur_bp->pt + newlen + MAX_GAP, (oldgap + del) - (MAX_GAP + newlen));
       cur_bp->gap = MAX_GAP;
     }
   else
@@ -164,14 +165,14 @@ replace_estr (size_t del, estr es)
 
   /* Zero any new bit of gap not produced by astr_insert. */
   if (MAX (oldgap, newlen) + added_gap < cur_bp->gap + newlen)
-    astr_set (cur_bp->text.as, cur_bp->pt + MAX (oldgap, newlen) + added_gap, '\0', newlen + cur_bp->gap - MAX (oldgap, newlen) - added_gap);
+    astr_set (estr_get_as (cur_bp->text), cur_bp->pt + MAX (oldgap, newlen) + added_gap, '\0', newlen + cur_bp->gap - MAX (oldgap, newlen) - added_gap);
 
   /* Insert `newlen' chars. */
   estr_replace_estr (cur_bp->text, cur_bp->pt, es);
   cur_bp->pt += newlen;
 
   /* Adjust markers. */
-  for (Marker *m = get_buffer_markers (cur_bp); m != NULL; m = get_marker_next (m))
+  for (Marker m = get_buffer_markers (cur_bp); m != NULL; m = get_marker_next (m))
     if (get_marker_o (m) > cur_bp->pt - newlen)
       set_marker_o (m, MAX (cur_bp->pt - newlen, get_marker_o (m) + newlen - del));
 
@@ -182,43 +183,43 @@ replace_estr (size_t del, estr es)
 }
 
 bool
-insert_estr (estr es)
+insert_estr (const_estr es)
 {
   return replace_estr (0, es);
 }
 
 char
-get_buffer_char (Buffer *bp, size_t o)
+get_buffer_char (Buffer bp, size_t o)
 {
-  return astr_get (bp->text.as, o_to_realo (bp, o));
+  return astr_get (estr_get_as (bp->text), o_to_realo (bp, o));
 }
 
 size_t
-buffer_prev_line (Buffer *bp, size_t o)
+buffer_prev_line (Buffer bp, size_t o)
 {
   return realo_to_o (bp, estr_prev_line (bp->text, o_to_realo (bp, o)));
 }
 
 size_t
-buffer_next_line (Buffer *bp, size_t o)
+buffer_next_line (Buffer bp, size_t o)
 {
   return realo_to_o (bp, estr_next_line (bp->text, o_to_realo (bp, o)));
 }
 
 size_t
-buffer_start_of_line (Buffer *bp, size_t o)
+buffer_start_of_line (Buffer bp, size_t o)
 {
   return realo_to_o (bp, estr_start_of_line (bp->text, o_to_realo (bp, o)));
 }
 
 size_t
-buffer_end_of_line (Buffer *bp, size_t o)
+buffer_end_of_line (Buffer bp, size_t o)
 {
   return realo_to_o (bp, estr_end_of_line (bp->text, o_to_realo (bp, o)));
 }
 
 size_t
-get_buffer_line_o (Buffer *bp)
+get_buffer_line_o (Buffer bp)
 {
   return realo_to_o (bp, estr_start_of_line (bp->text, o_to_realo (bp, bp->pt)));
 }
@@ -227,36 +228,34 @@ get_buffer_line_o (Buffer *bp)
 /* Buffer methods that don't know about the gap. */
 
 const char *
-get_buffer_eol (Buffer *bp)
+get_buffer_eol (Buffer bp)
 {
-  return bp->text.eol;
+  return estr_get_eol (bp->text);
 }
 
-/*
- * Copy a region of text into an allocated buffer.
- */
+/* Get the buffer region as an estr. */
 estr
-get_buffer_region (Buffer *bp, Region r)
+get_buffer_region (Buffer bp, Region r)
 {
   astr as = astr_new ();
-  if (r.start < get_buffer_pt (bp))
-    astr_cat (as, astr_substr (get_buffer_pre_point (bp), r.start, MIN (r.end, get_buffer_pt (bp)) - r.start));
-  if (r.end > get_buffer_pt (bp))
+  if (get_region_start (r) < bp->pt)
+    astr_cat (as, astr_substr (get_buffer_pre_point (bp), get_region_start (r), MIN (get_region_end (r), bp->pt) - get_region_start (r)));
+  if (get_region_end (r) > bp->pt)
     {
-      size_t from = MAX (r.start, get_buffer_pt (bp));
-      astr_cat (as, astr_substr (get_buffer_post_point (bp), from - get_buffer_pt (bp), r.end - from));
+      size_t from = MAX (get_region_start (r), bp->pt);
+      astr_cat (as, astr_substr (get_buffer_post_point (bp), from - bp->pt, get_region_end (r) - from));
     }
-  return (estr) {.as = as, .eol = get_buffer_eol (bp)};
+  return estr_new (as, get_buffer_eol (bp));
 }
 
 /*
  * Insert the character `c' at point in the current buffer.
  */
-int
+bool
 insert_char (int c)
 {
   const char ch = (char) c;
-  return replace_estr (false, (estr) {.as = astr_cat_nstr (astr_new (), &ch, 1), .eol = coding_eol_lf});
+  return insert_estr (estr_new (const_astr_new_nstr (&ch, 1), coding_eol_lf));
 }
 
 bool
@@ -287,23 +286,22 @@ delete_char (void)
 }
 
 void
-insert_buffer (Buffer * bp)
+insert_buffer (Buffer bp)
 {
   /* Copy text to avoid problems when bp == cur_bp. */
-  insert_estr ((estr) {.as = astr_cpy (astr_new (), get_buffer_pre_point (bp)), .eol = get_buffer_eol (bp)});
-  insert_estr ((estr) {.as = astr_cpy (astr_new (), get_buffer_post_point (bp)), .eol = get_buffer_eol (bp)});
+  insert_estr (estr_new (get_buffer_pre_point (bp), get_buffer_eol (bp)));
+  insert_estr (estr_new (get_buffer_post_point (bp), get_buffer_eol (bp)));
 }
 
 /*
  * Allocate a new buffer structure, set the default local
  * variable values, and insert it into the buffer list.
  */
-Buffer *
+Buffer
 buffer_new (void)
 {
-  Buffer *bp = (Buffer *) XZALLOC (Buffer);
-  bp->text.as = astr_new ();
-  bp->text.eol = coding_eol_lf;
+  Buffer bp = (Buffer) XZALLOC (struct Buffer);
+  bp->text = estr_new_astr (astr_new ());
   bp->dir = agetcwd ();
 
   /* Insert into buffer list. */
@@ -319,7 +317,7 @@ buffer_new (void)
  * Unchain the buffer's markers.
  */
 void
-free_buffer (Buffer * bp)
+destroy_buffer (Buffer bp)
 {
   while (bp->markers)
     unchain_marker (bp->markers);
@@ -329,7 +327,7 @@ free_buffer (Buffer * bp)
  * Initialise a buffer
  */
 void
-init_buffer (Buffer * bp)
+init_buffer (Buffer bp)
 {
   if (get_variable_bool ("auto-fill-mode"))
     set_buffer_autofill (bp, true);
@@ -339,7 +337,7 @@ init_buffer (Buffer * bp)
  * Get filename, or buffer name if NULL.
  */
 const char *
-get_buffer_filename_or_name (Buffer * bp)
+get_buffer_filename_or_name (Buffer bp)
 {
   const char *fname = get_buffer_filename (bp);
   return fname ? fname : get_buffer_name (bp);
@@ -349,7 +347,7 @@ get_buffer_filename_or_name (Buffer * bp)
  * Set a new filename, and from it a name, for the buffer.
  */
 void
-set_buffer_names (Buffer * bp, const char *filename)
+set_buffer_names (Buffer bp, const char *filename)
 {
   if (filename[0] != '/')
     filename = astr_cstr (astr_fmt ("%s/%s", astr_cstr (agetcwd ()), filename));
@@ -359,17 +357,17 @@ set_buffer_names (Buffer * bp, const char *filename)
   char *name = xstrdup (s);
   /* Note: there can't be more than SIZE_MAX buffers. */
   for (size_t i = 2; find_buffer (name) != NULL; i++)
-    name = xasprintf ("%s<%zd>", s, i);
+    name = xasprintf ("%s<%zu>", s, i);
   set_buffer_name (bp, name);
 }
 
 /*
  * Search for a buffer named `name'.
  */
-Buffer *
+Buffer
 find_buffer (const char *name)
 {
-  for (Buffer *bp = head_bp; bp != NULL; bp = bp->next)
+  for (Buffer bp = head_bp; bp != NULL; bp = bp->next)
     {
       const char *bname = get_buffer_name (bp);
       if (bname && STREQ (bname, name))
@@ -383,10 +381,10 @@ find_buffer (const char *name)
  * Move the given buffer to head.
  */
 static void
-move_buffer_to_head (Buffer * bp)
+move_buffer_to_head (Buffer bp)
 {
-  Buffer *prev = NULL;
-  for (Buffer *it = head_bp; it != bp; prev = it, it = it->next)
+  Buffer prev = NULL;
+  for (Buffer it = head_bp; it != bp; prev = it, it = it->next)
     ;
   if (prev)
     {
@@ -400,7 +398,7 @@ move_buffer_to_head (Buffer * bp)
  * Switch to the specified buffer.
  */
 void
-switch_to_buffer (Buffer * bp)
+switch_to_buffer (Buffer bp)
 {
   assert (get_window_bp (cur_wp) == cur_bp);
 
@@ -427,7 +425,7 @@ switch_to_buffer (Buffer * bp)
  * Print an error message into the echo area and return true
  * if the current buffer is readonly; otherwise return false.
  */
-int
+bool
 warn_if_readonly_buffer (void)
 {
   if (get_buffer_readonly (cur_bp))
@@ -439,7 +437,7 @@ warn_if_readonly_buffer (void)
   return false;
 }
 
-int
+bool
 warn_if_no_mark (void)
 {
   if (!cur_bp->mark)
@@ -455,58 +453,14 @@ warn_if_no_mark (void)
   return false;
 }
 
-/*
- * Make a region from two offsets.
- */
-Region
-region_new (size_t o1, size_t o2)
-{
-  return (Region) {.start = MIN (o1, o2), .end = MAX (o1, o2)};
-}
-
-size_t
-get_region_size (const Region r)
-{
-  return r.end - r.start;
-}
-
-/*
- * Return the region between point and mark.
- */
-Region
-calculate_the_region (void)
-{
-  return region_new (cur_bp->pt, get_marker_o (cur_bp->mark));
-}
-
-bool
-delete_region (const Region r)
-{
-  if (warn_if_readonly_buffer ())
-    return false;
-
-  Marker *m = point_marker ();
-  goto_offset (r.start);
-  replace_estr (get_region_size (r), estr_empty);
-  goto_offset (get_marker_o (m));
-  unchain_marker (m);
-  return true;
-}
-
-bool
-in_region (size_t o, size_t x, Region r)
-{
-  return o + x >= r.start && o + x < r.end;
-}
-
 /*
  * Set the specified buffer temporary flag and move the buffer
  * to the end of the buffer list.
  */
 void
-set_temporary_buffer (Buffer * bp)
+set_temporary_buffer (Buffer bp)
 {
-  Buffer *bp0;
+  Buffer bp0;
 
   set_buffer_temporary (bp, true);
 
@@ -549,15 +503,19 @@ deactivate_mark (void)
  * Return a safe tab width for the given buffer.
  */
 size_t
-tab_width (Buffer * bp)
+tab_width (Buffer bp)
 {
-  return MAX (get_variable_number_bp (bp, "tab-width"), 1);
+  long res = 0;
+  lisp_to_number (get_variable_bp (bp, "tab-width"), &res);
+  if (res < 1)
+    res = 8;
+  return res;
 }
 
-Buffer *
+Buffer
 create_auto_buffer (const char *name)
 {
-  Buffer *bp = buffer_new ();
+  Buffer bp = buffer_new ();
   set_buffer_name (bp, name);
   set_buffer_needname (bp, true);
   set_buffer_temporary (bp, true);
@@ -565,7 +523,7 @@ create_auto_buffer (const char *name)
   return bp;
 }
 
-Buffer *
+Buffer
 create_scratch_buffer (void)
 {
   return create_auto_buffer ("*scratch*");
@@ -576,9 +534,9 @@ create_scratch_buffer (void)
  * its space.  Recreate the scratch buffer when required.
  */
 void
-kill_buffer (Buffer * kill_bp)
+kill_buffer (Buffer kill_bp)
 {
-  Buffer *next_bp;
+  Buffer next_bp;
 
   if (get_buffer_next (kill_bp) != NULL)
     next_bp = get_buffer_next (kill_bp);
@@ -586,12 +544,12 @@ kill_buffer (Buffer * kill_bp)
     next_bp = (head_bp == kill_bp) ? NULL : head_bp;
 
   /* Search for windows displaying the buffer to kill. */
-  for (Window *wp = head_wp; wp != NULL; wp = get_window_next (wp))
+  for (Window wp = head_wp; wp != NULL; wp = get_window_next (wp))
     if (get_window_bp (wp) == kill_bp)
       {
         set_window_bp (wp, next_bp);
         set_window_topdelta (wp, 0);
-        set_window_saved_pt (wp, NULL); /* The old marker will be freed. */
+        set_window_saved_pt (wp, NULL);
       }
 
   /* Remove the buffer from the buffer list. */
@@ -599,26 +557,26 @@ kill_buffer (Buffer * kill_bp)
     cur_bp = next_bp;
   if (head_bp == kill_bp)
     head_bp = get_buffer_next (head_bp);
-  for (Buffer *bp = head_bp; bp != NULL && get_buffer_next (bp) != NULL; bp = get_buffer_next (bp))
+  for (Buffer bp = head_bp; bp != NULL && get_buffer_next (bp) != NULL; bp = get_buffer_next (bp))
     if (get_buffer_next (bp) == kill_bp)
       {
         set_buffer_next (bp, get_buffer_next (get_buffer_next (bp)));
         break;
       }
 
-  free_buffer (kill_bp);
+  destroy_buffer (kill_bp);
 
   /* If no buffers left, recreate scratch buffer and point windows at
      it. */
   if (next_bp == NULL)
     {
       cur_bp = head_bp = next_bp = create_scratch_buffer ();
-      for (Window *wp = head_wp; wp != NULL; wp = get_window_next (wp))
+      for (Window wp = head_wp; wp != NULL; wp = get_window_next (wp))
         set_window_bp (wp, head_bp);
     }
 
   /* Resync windows that need it. */
-  for (Window *wp = head_wp; wp != NULL; wp = get_window_next (wp))
+  for (Window wp = head_wp; wp != NULL; wp = get_window_next (wp))
     if (get_window_bp (wp) == next_bp)
       window_resync (wp);
 }
@@ -630,12 +588,12 @@ Kill buffer BUFFER.
 With a nil argument, kill the current buffer.
 +*/
 {
-  Buffer *bp;
+  Buffer bp;
 
   STR_INIT (buf)
   else
     {
-      Completion *cp = make_buffer_completion ();
+      Completion cp = make_buffer_completion ();
       buf = minibuf_read_completion ("Kill buffer (default %s): ",
                                      "", cp, NULL, get_buffer_name (cur_bp));
       if (buf == NULL)
@@ -664,11 +622,11 @@ With a nil argument, kill the current buffer.
 }
 END_DEFUN
 
-Completion *
+Completion
 make_buffer_completion (void)
 {
-  Completion *cp = completion_new (false);
-  for (Buffer *bp = head_bp; bp != NULL; bp = get_buffer_next (bp))
+  Completion cp = completion_new (false);
+  for (Buffer bp = head_bp; bp != NULL; bp = get_buffer_next (bp))
     gl_sortedlist_add (get_completion_completions (cp), completion_strcmp,
                        xstrdup (get_buffer_name (bp)));
 
@@ -681,7 +639,7 @@ make_buffer_completion (void)
  * true, else false.
  */
 bool
-check_modified_buffer (Buffer * bp)
+check_modified_buffer (Buffer bp)
 {
   if (get_buffer_modified (bp) && !get_buffer_nosave (bp))
     for (;;)
@@ -705,7 +663,7 @@ check_modified_buffer (Buffer * bp)
 /* Basic movement routines */
 
 bool
-move_char (int offset)
+move_char (ptrdiff_t offset)
 {
   int dir = offset >= 0 ? 1 : -1;
   for (size_t i = 0; i < (size_t) (abs (offset)); i++)
@@ -751,9 +709,9 @@ goto_goalc (void)
 }
 
 bool
-move_line (int n)
+move_line (ptrdiff_t n)
 {
-  size_t (*func) (Buffer *bp, size_t o) = buffer_next_line;
+  size_t (*func) (Buffer bp, size_t o) = buffer_next_line;
   if (n < 0)
     {
       n = -n;
@@ -778,7 +736,7 @@ move_line (int n)
 }
 
 size_t
-offset_to_line (Buffer *bp, size_t offset)
+offset_to_line (Buffer bp, size_t offset)
 {
   size_t n = 0;
   for (size_t o = 0; buffer_end_of_line (bp, o) < offset; o = buffer_next_line (bp, o))
