@@ -1,6 +1,6 @@
 /* Disk file handling
 
-   Copyright (c) 1997-2012 Free Software Foundation, Inc.
+   Copyright (c) 1997-2014 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -37,7 +37,7 @@
 #include "main.h"
 #include "extern.h"
 
-int
+bool
 exist_file (const char *filename)
 {
   struct stat st;
@@ -203,7 +203,7 @@ check_writable (const char *filename)
 bool
 find_file (const char *filename)
 {
-  Buffer *bp;
+  Buffer bp;
   for (bp = head_bp; bp != NULL; bp = get_buffer_next (bp))
     if (get_buffer_filename (bp) != NULL &&
         STREQ (get_buffer_filename (bp), filename))
@@ -223,10 +223,10 @@ find_file (const char *filename)
           set_buffer_dir (bp, astr_new_cstr (dir_name (filename)));
 
           estr es = estr_readf (filename);
-          if (es.as)
+          if (es)
             set_buffer_readonly (bp, !check_writable (filename));
           else
-            es.as = astr_new ();
+            es = estr_new_astr (astr_new ());
           set_buffer_text (bp, es);
 
           /* Reset undo history. */
@@ -294,7 +294,7 @@ If the current buffer now contains an empty file that you just visited
     buf = astr_cstr (get_buffer_dir (cur_bp));
   else
     base = base_name (buf);
-  castr ms = minibuf_read_filename ("Find alternate: ", buf, base);
+  const_astr ms = minibuf_read_filename ("Find alternate: ", buf, base);
 
   ok = leNIL;
   if (ms == NULL)
@@ -313,12 +313,12 @@ DEFUN_ARGS ("switch-to-buffer", switch_to_buffer,
 Select buffer @i{buffer} in the current window.
 +*/
 {
-  Buffer *bp = ((get_buffer_next (cur_bp) != NULL) ? get_buffer_next (cur_bp) : head_bp);
+  Buffer bp = ((get_buffer_next (cur_bp) != NULL) ? get_buffer_next (cur_bp) : head_bp);
 
   STR_INIT (buf)
   else
     {
-      Completion *cp = make_buffer_completion ();
+      Completion cp = make_buffer_completion ();
       buf = minibuf_read_completion ("Switch to buffer (default %s): ",
                                      "", cp, NULL, get_buffer_name (bp));
 
@@ -352,7 +352,7 @@ Insert after point the contents of BUFFER.
 Puts mark after the inserted text.
 +*/
 {
-  Buffer *def_bp = ((get_buffer_next (cur_bp) != NULL) ? get_buffer_next (cur_bp) : head_bp);
+  Buffer def_bp = ((get_buffer_next (cur_bp) != NULL) ? get_buffer_next (cur_bp) : head_bp);
 
   if (warn_if_readonly_buffer ())
     return leNIL;
@@ -360,7 +360,7 @@ Puts mark after the inserted text.
   STR_INIT (buf)
   else
     {
-      Completion *cp = make_buffer_completion ();
+      Completion cp = make_buffer_completion ();
       buf = minibuf_read_completion ("Insert buffer (default %s): ",
                                      "", cp, NULL, get_buffer_name (def_bp));
       if (buf == NULL)
@@ -369,7 +369,7 @@ Puts mark after the inserted text.
 
   if (ok == leT)
     {
-      Buffer *bp;
+      Buffer bp;
 
       if (buf && astr_len (buf) > 0)
         {
@@ -417,7 +417,7 @@ Set mark after the inserted text.
   if (ok != leNIL)
     {
       estr es = estr_readf (astr_cstr (file));
-      if (es.as != NULL)
+      if (es)
         {
           insert_estr (es);
           FUNCALL (set_mark_command);
@@ -435,14 +435,14 @@ END_DEFUN
  * Write buffer to given file name with given mode.
  */
 static int
-write_to_disk (Buffer * bp, const char *filename, mode_t mode)
+write_to_disk (Buffer bp, const char *filename, mode_t mode)
 {
   int fd = creat (filename, mode);
   if (fd < 0)
     return -1;
 
   int ret = 0;
-  castr as = get_buffer_pre_point (bp);
+  const_astr as = get_buffer_pre_point (bp);
   ssize_t written = write (fd, astr_cstr (as), astr_len (as));
   if (written < 0 || (size_t) written != astr_len (as))
     ret = written;
@@ -495,7 +495,7 @@ create_backup_filename (const char *filename, const char *backupdir)
  * Create a backup file if specified by the user variables.
  */
 static int
-backup_and_write (Buffer * bp, const char *filename)
+backup_and_write (Buffer bp, const char *filename)
 {
   /* Make backup of original file. */
   int fd, backup = get_variable_bool ("make-backup-files");
@@ -529,19 +529,19 @@ backup_and_write (Buffer * bp, const char *filename)
 }
 
 static le *
-write_buffer (Buffer *bp, bool needname, bool confirm,
+write_buffer (Buffer bp, bool needname, bool confirm,
               const char *name0, const char *prompt)
 {
   bool ans = true;
   le * ok = leT;
-  castr name;
+  const_astr name;
 
   if (!needname)
     name = astr_new_cstr (name0);
 
   if (needname)
     {
-      name = minibuf_read_filename (prompt, "", NULL);
+      name = minibuf_read_filename ("%s", "", NULL, prompt);
       if (name == NULL)
         return FUNCALL (keyboard_quit);
       if (astr_len (name) == 0)
@@ -551,7 +551,31 @@ write_buffer (Buffer *bp, bool needname, bool confirm,
 
   if (confirm && exist_file (astr_cstr (name)))
     {
-      ans = minibuf_read_yn ("File `%s' exists; overwrite? (y or n) ", astr_cstr (name));
+      char *buf = xasprintf ("File `%s' exists; overwrite? (y or n) ", astr_cstr (name));
+      const char *errmsg = "";
+
+      for (ans = -2; ans == -2;) {
+        minibuf_write ("%s%s", errmsg, buf);
+        switch (getkeystroke (GETKEY_DEFAULT))
+          {
+          case 'y':
+          case 'Y':
+          case ' ':
+          case KBD_RET:
+            ans = true;
+            break;
+          case 'N':
+          case 'n':
+          case KBD_DEL:
+            ans = false;
+            break;
+          case KBD_CTRL | 'g':
+            ans = -1;
+          default:
+            errmsg = "Please answer y or n.  ";
+          }
+      }
+
       if (ans == -1)
         FUNCALL (keyboard_quit);
       else if (ans == false)
@@ -582,7 +606,7 @@ write_buffer (Buffer *bp, bool needname, bool confirm,
 }
 
 static le *
-save_buffer (Buffer * bp)
+save_buffer (Buffer bp)
 {
   if (get_buffer_modified (bp))
     return write_buffer (bp, get_buffer_needname (bp), false, get_buffer_filename (bp),
@@ -622,7 +646,7 @@ save_some_buffers (void)
   bool none_to_save = true;
   bool noask = false;
 
-  for (Buffer *bp = head_bp; bp != NULL; bp = get_buffer_next (bp))
+  for (Buffer bp = head_bp; bp != NULL; bp = get_buffer_next (bp))
     {
       if (get_buffer_modified (bp) && !get_buffer_nosave (bp))
         {
@@ -698,7 +722,7 @@ Offer to save each buffer, then kill this Zile process.
   if (!save_some_buffers ())
     return leNIL;
 
-  for (Buffer *bp = head_bp; bp != NULL; bp = get_buffer_next (bp))
+  for (Buffer bp = head_bp; bp != NULL; bp = get_buffer_next (bp))
     if (get_buffer_modified (bp) && !get_buffer_needname (bp))
       {
         for (;;)
@@ -725,10 +749,10 @@ END_DEFUN
  * otherwise, exit.
  */
 void
-zile_exit (int doabort)
+zile_exit (bool doabort)
 {
   fprintf (stderr, "Trying to save modified buffers (if any)...\r\n");
-  for (Buffer *bp = head_bp; bp != NULL; bp = get_buffer_next (bp))
+  for (Buffer bp = head_bp; bp != NULL; bp = get_buffer_next (bp))
     if (get_buffer_modified (bp) && !get_buffer_nosave (bp))
       {
         astr buf = astr_fmt ("%s.%sSAVE",
@@ -767,7 +791,7 @@ Make DIR become the current buffer's default directory.
         }
       else if (chdir (astr_cstr (dir)) == -1)
         {
-          minibuf_write ("%s: %s", dir, strerror (errno));
+          minibuf_write ("%s: %s", astr_cstr (dir), strerror (errno));
           ok = leNIL;
         }
     }
