@@ -1,6 +1,6 @@
 /* Miscellaneous Emacs functions
 
-   Copyright (c) 1997-2011 Free Software Foundation, Inc.
+   Copyright (c) 1997-2014 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -58,14 +58,14 @@ void
 write_temp_buffer (const char *name, bool show, void (*func) (va_list ap), ...)
 {
   /* Popup a window with the buffer "name". */
-  Window *old_wp = cur_wp;
-  Buffer *old_bp = cur_bp;
-  Window *wp = find_window (name);
+  Window old_wp = cur_wp;
+  Buffer old_bp = cur_bp;
+  Window wp = find_window (name);
   if (show && wp)
     set_current_window (wp);
   else
     {
-      Buffer *bp = find_buffer (name);
+      Buffer bp = find_buffer (name);
       if (show)
         set_current_window (popup_window ());
       if (bp == NULL)
@@ -77,7 +77,7 @@ write_temp_buffer (const char *name, bool show, void (*func) (va_list ap), ...)
     }
 
   /* Remove the contents of that buffer. */
-  Buffer *new_bp = buffer_new ();
+  Buffer new_bp = buffer_new ();
   set_buffer_name (new_bp, get_buffer_name (cur_bp));
   kill_buffer (cur_bp);
   cur_bp = new_bp;
@@ -110,8 +110,8 @@ write_temp_buffer (const char *name, bool show, void (*func) (va_list ap), ...)
 static void
 write_buffers_list (va_list ap)
 {
-  Window *old_wp = va_arg (ap, Window *);
-  Buffer *bp;
+  Window old_wp = va_arg (ap, Window);
+  Buffer bp;
 
   /* FIXME: Underline next line properly. */
   bprintf ("CRM Buffer                Size  Mode             File\n");
@@ -123,9 +123,9 @@ write_buffers_list (va_list ap)
     {
       /* Print all buffers whose names don't start with space except
          this one (the *Buffer List*). */
-      if (cur_bp != bp && get_buffer_name (bp)[0] == ' ')
+      if (cur_bp != bp && get_buffer_name (bp)[0] != ' ')
         {
-          bprintf ("%c%c%c %-19s %6u  %-17s",
+          bprintf ("%c%c%c %-19s %6zu  %-17s",
                    get_window_bp (old_wp) == bp ? '.' : ' ',
                    get_buffer_readonly (bp) ? '%' : ' ',
                    get_buffer_modified (bp) ? '*' : ' ',
@@ -190,7 +190,7 @@ Just C-u as argument means to use the current column.
 
   if (!(lastflag & FLAG_SET_UNIARG) && arglist == NULL)
     {
-      fill_col = minibuf_read_number ("Set fill-column to (default %d): ", o);
+      fill_col = minibuf_read_number ("Set fill-column to (default %zu): ", o);
       if (fill_col == LONG_MAX)
         return leNIL;
       else if (fill_col == LONG_MAX - 1)
@@ -211,8 +211,8 @@ Just C-u as argument means to use the current column.
     {
       buf = xasprintf ("%ld", fill_col);
       /* Only print message when run interactively. */
-      minibuf_write ("Fill column set to %s (was %d)", buf,
-                     get_variable_number ("fill-column"));
+      minibuf_write ("Fill column set to %s (was %s)", buf,
+                     get_variable ("fill-column"));
     }
 
   if (ok == leT)
@@ -388,7 +388,7 @@ iswordchar (int c)
 }
 
 static bool
-move_word (int dir)
+move_word (ptrdiff_t dir)
 {
   bool gotword = false;
   do
@@ -446,7 +446,7 @@ isclosebracketchar (int c, bool single_quote, bool double_quote)
 }
 
 static bool
-move_sexp (int dir)
+move_sexp (ptrdiff_t dir)
 {
   int gotsexp = false, level = 0;
   int single_quote = dir < 0, double_quote = single_quote;
@@ -559,11 +559,11 @@ static void
 astr_append_region (astr s)
 {
   activate_mark ();
-  astr_cat (s, get_buffer_region (cur_bp, calculate_the_region ()).as);
+  astr_cat (s, estr_get_as (get_buffer_region (cur_bp, calculate_the_region ())));
 }
 
 static bool
-transpose_subr (bool (*move_func) (int dir))
+transpose_subr (bool (*move_func) (ptrdiff_t dir))
 {
   /* For transpose-chars. */
   if (move_func == move_char && eolp ())
@@ -582,7 +582,7 @@ transpose_subr (bool (*move_func) (int dir))
 
   /* Mark the beginning of first string. */
   push_mark ();
-  Marker *m1 = point_marker ();
+  Marker m1 = point_marker ();
 
   /* Check to make sure we can go forwards twice. */
   if (!move_func (1) || !move_func (1))
@@ -619,7 +619,7 @@ transpose_subr (bool (*move_func) (int dir))
 
   /* For transpose-lines. */
   astr as2 = NULL;
-  Marker *m2;
+  Marker m2;
   if (move_func == move_line)
     m2 = point_marker ();
   else
@@ -662,16 +662,14 @@ transpose_subr (bool (*move_func) (int dir))
 }
 
 static le *
-transpose (int uniarg, bool (*move) (int dir))
+transpose (int uniarg, bool (*move) (ptrdiff_t dir))
 {
   if (warn_if_readonly_buffer ())
     return leNIL;
 
   bool ret = true;
-  undo_start_sequence ();
   for (unsigned long uni = 0; ret && uni < (unsigned) abs (uniarg); ++uni)
     ret = transpose_subr (move);
-  undo_end_sequence ();
 
   return bool_to_lisp (ret);
 }
@@ -839,14 +837,12 @@ DEFUN ("fill-paragraph", fill_paragraph)
 Fill paragraph at or after point.
 +*/
 {
-  Marker *m = point_marker ();
-
-  undo_start_sequence ();
+  Marker m = point_marker ();
 
   FUNCALL (forward_paragraph);
   if (is_empty_line ())
     previous_line ();
-  Marker *m_end = point_marker ();
+  Marker m_end = point_marker ();
 
   FUNCALL (backward_paragraph);
   if (is_empty_line ())
@@ -862,14 +858,14 @@ Fill paragraph at or after point.
   unchain_marker (m_end);
 
   FUNCALL (end_of_line);
-  while (get_goalc () > (size_t) get_variable_number ("fill-column") + 1
-         && fill_break_line ())
+  int ret;
+  while ((ret = fill_break_line ()) == true)
     ;
+  if (ret == -1)
+    ok = leNIL;
 
   goto_offset (get_marker_o (m));
   unchain_marker (m);
-
-  undo_end_sequence ();
 }
 END_DEFUN
 
@@ -912,7 +908,7 @@ Convert following word (or @i{arg} words) to lower case, moving over.
 +*/
 {
   INT_OR_UNIARG_INIT (arg);
-  ok = execute_with_uniarg (true, arg, setcase_word_lowercase, NULL);
+  ok = execute_with_uniarg (arg, setcase_word_lowercase, NULL);
 }
 END_DEFUN
 
@@ -929,7 +925,7 @@ Convert following word (or @i{arg} words) to upper case, moving over.
 +*/
 {
   INT_OR_UNIARG_INIT (arg);
-  ok = execute_with_uniarg (true, arg, setcase_word_uppercase, NULL);
+  ok = execute_with_uniarg (arg, setcase_word_uppercase, NULL);
 }
 END_DEFUN
 
@@ -948,7 +944,7 @@ and the rest lower case.
 +*/
 {
   INT_OR_UNIARG_INIT (arg);
-  ok = execute_with_uniarg (true, arg, setcase_word_capitalize, NULL);
+  ok = execute_with_uniarg (arg, setcase_word_capitalize, NULL);
 }
 END_DEFUN
 
@@ -962,10 +958,8 @@ setcase_region (int (*func) (int))
     return leNIL;
 
   Region r = calculate_the_region ();
-  undo_start_sequence ();
-
-  Marker *m = point_marker ();
-  goto_offset (r.start);
+  Marker m = point_marker ();
+  goto_offset (get_region_start (r));
   for (size_t size = get_region_size (r); size > 0; size--)
     {
       char c = func (following_char ());
@@ -975,8 +969,6 @@ setcase_region (int (*func) (int))
   goto_offset (get_marker_o (m));
   unchain_marker (m);
 
-  undo_end_sequence ();
-
   return leT;
 }
 
@@ -1001,7 +993,7 @@ END_DEFUN
 static void
 write_shell_output (va_list ap)
 {
-  insert_estr ((estr) {.as = va_arg (ap, astr), .eol = coding_eol_lf});
+  insert_estr (estr_new (va_arg (ap, astr), coding_eol_lf));
 }
 
 typedef struct {
@@ -1038,7 +1030,7 @@ done_read (void *data, size_t n, void *priv)
 }
 
 static le *
-pipe_command (castr cmd, astr input, bool do_insert, bool do_replace)
+pipe_command (const_astr cmd, astr input, bool do_insert, bool do_replace)
 {
   const char *prog_argv[] = { "/bin/sh", "-c", astr_cstr (cmd), NULL };
   pipe_data inout = { .in = input, .out = astr_new (), .done = 0 };
@@ -1059,7 +1051,7 @@ pipe_command (castr cmd, astr input, bool do_insert, bool do_replace)
           if (do_replace && !warn_if_no_mark ())
             {
               Region r = calculate_the_region ();
-              goto_offset (r.start);
+              goto_offset (get_region_start (r));
               del = get_region_size (r);
             }
           replace_estr (del, estr_new_astr (inout.out));
@@ -1078,10 +1070,10 @@ pipe_command (castr cmd, astr input, bool do_insert, bool do_replace)
   return leT;
 }
 
-static castr
+static const_astr
 minibuf_read_shell_command (void)
 {
-  castr ms = minibuf_read ("Shell command: ", "");
+  const_astr ms = minibuf_read ("Shell command: ", "");
 
   if (ms == NULL)
     {
@@ -1124,12 +1116,16 @@ says to insert the output in the current buffer.
 END_DEFUN
 
 /* The `start' and `end' arguments are fake, hence their string type,
-   so they can be ignored. */
+   so they can be ignored, hence the pragma (FIXME: work out how to
+   apply just to `start' and `end'). */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-but-set-variable"
 DEFUN_ARGS ("shell-command-on-region", shell_command_on_region,
             STR_ARG (start)
             STR_ARG (end)
             STR_ARG (cmd)
             BOOL_ARG (insert))
+#pragma GCC diagnostic pop
 /*+
 Execute string command in inferior shell with region as input.
 Normally display output (if any) in temp buffer `*Shell Command Output*';
@@ -1157,7 +1153,7 @@ The output is available in that buffer in both cases.
       if (warn_if_no_mark ())
         ok = leNIL;
       else
-        ok = pipe_command (cmd, get_buffer_region (cur_bp, calculate_the_region ()).as, insert, true);
+        ok = pipe_command (cmd, estr_get_as (get_buffer_region (cur_bp, calculate_the_region ())), insert, true);
     }
 }
 END_DEFUN
@@ -1181,20 +1177,17 @@ On isolated blank line, delete that one.
 On nonblank line, delete any immediately following blank lines.
 +*/
 {
-  Marker *m = point_marker ();
-  Region r;
-  r.start = r.end = get_buffer_line_o (cur_bp);
-
-  undo_start_sequence ();
+  Marker m = point_marker ();
+  Region r = region_new (get_buffer_line_o (cur_bp), get_buffer_line_o (cur_bp));
 
   /* Find following blank lines.  */
   if (FUNCALL (forward_line) == leT && is_blank_line ())
     {
-      r.start = get_buffer_pt (cur_bp);
+      set_region_start (r, get_buffer_pt (cur_bp));
       do
-        r.end = buffer_next_line (cur_bp, get_buffer_pt (cur_bp));
+        set_region_end (r, buffer_next_line (cur_bp, get_buffer_pt (cur_bp)));
       while (FUNCALL (forward_line) == leT && is_blank_line ());
-      r.end = MIN (r.end, get_buffer_size (cur_bp));
+      set_region_end (r, MIN (get_region_end (r), get_buffer_size (cur_bp)));
     }
   goto_offset (get_marker_o (m));
 
@@ -1202,24 +1195,24 @@ On nonblank line, delete any immediately following blank lines.
   bool singleblank = true;
   if (is_blank_line ())
     {
-      r.end = MAX (r.end, buffer_next_line (cur_bp, get_buffer_pt (cur_bp)));
+      set_region_end (r, MAX (get_region_end (r), buffer_next_line (cur_bp, get_buffer_pt (cur_bp))));
       do
-        r.start = get_buffer_line_o (cur_bp);
+        set_region_start (r, get_buffer_line_o (cur_bp));
       while (FUNCALL_ARG (forward_line, -1) == leT && is_blank_line ());
       goto_offset (get_marker_o (m));
-      if (r.start != get_buffer_line_o (cur_bp) ||
-          r.end > buffer_next_line (cur_bp, get_buffer_pt (cur_bp)))
+      if (get_region_start (r) != get_buffer_line_o (cur_bp) ||
+          get_region_end (r) > buffer_next_line (cur_bp, get_buffer_pt (cur_bp)))
         singleblank = false;
-      r.end = MIN (r.end, get_buffer_size (cur_bp));
+      set_region_end (r, MIN (get_region_end (r), get_buffer_size (cur_bp)));
     }
 
   /* If we are deleting to EOB, need to fudge extra line. */
-  bool at_eob = r.end == get_buffer_size (cur_bp) && r.start > 0;
+  bool at_eob = get_region_end (r) == get_buffer_size (cur_bp) && get_region_start (r) > 0;
   if (at_eob)
-    r.start -= strlen (get_buffer_eol (cur_bp));
+    set_region_start (r, get_region_start (r) - strlen (get_buffer_eol (cur_bp)));
 
   /* Delete any blank lines found. */
-  if (r.start < r.end)
+  if (get_region_start (r) < get_region_end (r))
     delete_region (r);
 
   /* If we found more than one blank line, leave one. */
@@ -1231,8 +1224,6 @@ On nonblank line, delete any immediately following blank lines.
         insert_newline ();
     }
 
-  undo_end_sequence ();
-
   unchain_marker (m);
   deactivate_mark ();
 }
