@@ -1,6 +1,6 @@
 /* Kill ring facility functions
 
-   Copyright (c) 2001, 2003-2012 Free Software Foundation, Inc.
+   Copyright (c) 2001-2014 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -31,17 +31,17 @@
 static estr kill_ring_text;
 
 static void
-maybe_free_kill_ring (void)
+maybe_destroy_kill_ring (void)
 {
   if (last_command () != F_kill_region)
-    kill_ring_text.as = NULL;
+    kill_ring_text = NULL;
 }
 
 static void
 kill_ring_push (estr es)
 {
-  if (kill_ring_text.as == NULL)
-    kill_ring_text = (estr) {.as = astr_new (), .eol = coding_eol_lf};
+  if (kill_ring_text == NULL)
+    kill_ring_text = estr_new_astr (astr_new ());
   estr_cat (kill_ring_text, es);
 }
 
@@ -90,8 +90,6 @@ kill_line (bool whole_line)
       return false;
     }
 
-  undo_start_sequence ();
-
   if (!eolp ())
     ok = copy_or_kill_region (true, region_new (get_buffer_pt (cur_bp), get_buffer_line_o (cur_bp) + cur_line_len));
 
@@ -100,12 +98,10 @@ kill_line (bool whole_line)
       if (FUNCALL (delete_char) == leNIL)
         return false;
 
-      kill_ring_push ((estr) {.as = astr_new_cstr ("\n"), .eol = coding_eol_lf});
+      kill_ring_push (estr_new_astr (astr_new_cstr ("\n")));
       set_this_command (F_kill_region);
     }
 
-  undo_end_sequence ();
-
   return ok;
 }
 
@@ -134,7 +130,7 @@ including its terminating newline, when used at the beginning of a line
 with no argument.
 +*/
 {
-  maybe_free_kill_ring ();
+  maybe_destroy_kill_ring ();
 
   INT_OR_UNIARG_INIT (arg);
 
@@ -142,13 +138,11 @@ with no argument.
     ok = bool_to_lisp (kill_line (bolp () && get_variable_bool ("kill-whole-line")));
   else
     {
-      undo_start_sequence ();
       if (arg <= 0)
         ok = bool_to_lisp (bolp () ||
                            copy_or_kill_region (true, region_new (get_buffer_line_o (cur_bp), get_buffer_pt (cur_bp))));
       if (arg != 0 && ok == leT)
-        ok = execute_with_uniarg (false, arg, kill_whole_line, kill_line_backward);
-      undo_end_sequence ();
+        ok = execute_with_uniarg (arg, kill_whole_line, kill_line_backward);
     }
 
   deactivate_mark ();
@@ -163,7 +157,7 @@ copy_or_kill_the_region (bool kill)
   if (!warn_if_no_mark ())
     {
       Region r = calculate_the_region ();
-      maybe_free_kill_ring ();
+      maybe_destroy_kill_ring ();
       ok = copy_or_kill_region (kill, r);
     }
 
@@ -199,20 +193,18 @@ END_DEFUN
 static le *
 kill_text (int uniarg, Function mark_func)
 {
-  maybe_free_kill_ring ();
+  maybe_destroy_kill_ring ();
 
   if (warn_if_readonly_buffer ())
     return leNIL;
 
   push_mark ();
-  undo_start_sequence ();
   mark_func (uniarg, true, NULL);
   FUNCALL (kill_region);
-  undo_end_sequence ();
   pop_mark ();
 
   set_this_command (F_kill_region);
-  minibuf_write ("");		/* Erase "Set mark" message.  */
+  minibuf_write ("%s", "");		/* Erase "Set mark" message.  */
   return leT;
 }
 
@@ -258,7 +250,7 @@ More precisely, reinsert the stretch of killed text most recently
 killed @i{or} yanked.  Put point at end, and set mark at beginning.
 +*/
 {
-  if (kill_ring_text.as == NULL)
+  if (kill_ring_text == NULL)
     {
       minibuf_error ("Kill ring is empty");
       return leNIL;
