@@ -1,6 +1,6 @@
 /* Line-oriented editing functions
 
-   Copyright (c) 1997-2011 Free Software Foundation, Inc.
+   Copyright (c) 1997-2014 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -34,7 +34,7 @@ static void
 insert_expanded_tab (void)
 {
   size_t t = tab_width (cur_bp);
-  bprintf ("%*s", t - get_goalc () % t);
+  bprintf ("%*s", (int) (t - get_goalc () % t), "");
 }
 
 static bool
@@ -57,7 +57,7 @@ Insert a tabulation at the current point position into the current
 buffer.
 +*/
 {
-  ok = execute_with_uniarg (true, uniarg, insert_tab, NULL);
+  ok = execute_with_uniarg (uniarg, insert_tab, NULL);
 }
 END_DEFUN
 
@@ -78,11 +78,18 @@ intercalate_newline (void)
  *
  * Return flag indicating whether break was made.
  */
-bool
+int
 fill_break_line (void)
 {
-  size_t fillcol = get_variable_number ("fill-column");
-  bool break_made = false;
+  long n;
+  if (!lisp_to_number (get_variable_bp (cur_bp, "fill-column"), &n) || n < 0)
+    {
+      minibuf_error ("Wrong type argument: number-or-markerp, nil");
+      return -1;
+    }
+  size_t fillcol = (size_t)n;
+
+  int break_made = false;
 
   /* Only break if we're beyond fill-column. */
   if (get_goalc () > fillcol)
@@ -90,7 +97,7 @@ fill_break_line (void)
       size_t break_col = 0;
 
       /* Save point. */
-      Marker *m = point_marker ();
+      Marker m = point_marker ();
 
       /* Move cursor back to fill column */
       size_t old_col = get_buffer_pt (cur_bp) - get_buffer_line_o (cur_bp);
@@ -139,10 +146,10 @@ fill_break_line (void)
 static bool
 newline (void)
 {
-  if (get_buffer_autofill (cur_bp) &&
-      get_goalc () > (size_t) get_variable_number ("fill-column"))
-    fill_break_line ();
-  return insert_newline ();
+  bool ret = true;
+  if (get_buffer_autofill (cur_bp))
+    ret = fill_break_line () != -1;
+  return ret ? insert_newline () : false;
 }
 
 DEFUN ("newline", newline)
@@ -151,7 +158,7 @@ Insert a newline at the current point position into
 the current buffer.
 +*/
 {
-  ok = execute_with_uniarg (true, uniarg, newline, NULL);
+  ok = execute_with_uniarg (uniarg, newline, NULL);
 }
 END_DEFUN
 
@@ -160,14 +167,14 @@ DEFUN ("open-line", open_line)
 Insert a newline and leave point before it.
 +*/
 {
-  ok = execute_with_uniarg (true, uniarg, intercalate_newline, NULL);
+  ok = execute_with_uniarg (uniarg, intercalate_newline, NULL);
 }
 END_DEFUN
 
 bool
 insert_newline (void)
 {
-  return insert_estr ((estr) {.as = astr_new_cstr ("\n"), .eol = coding_eol_lf});
+  return insert_estr (estr_new_astr (astr_new_cstr ("\n")));
 }
 
 DEFUN_NONINTERACTIVE_ARGS ("insert", insert,
@@ -187,7 +194,7 @@ bprintf (const char *fmt, ...)
   va_list ap;
 
   va_start (ap, fmt);
-  insert_estr ((estr) {.as = astr_vfmt (fmt, ap), .eol = coding_eol_lf});
+  insert_estr (estr_new (astr_vfmt (fmt, ap), coding_eol_lf));
   va_end (ap);
 }
 
@@ -213,7 +220,7 @@ Delete the following @i{n} characters (previous if @i{n} is negative).
 +*/
 {
   INT_OR_UNIARG_INIT (n);
-  ok = execute_with_uniarg (true, n, delete_char, backward_delete_char);
+  ok = execute_with_uniarg (n, delete_char, backward_delete_char);
 }
 END_DEFUN
 
@@ -224,7 +231,7 @@ Delete the previous @i{n} characters (following if @i{n} is negative).
 +*/
 {
   INT_OR_UNIARG_INIT (n);
-  ok = execute_with_uniarg (true, n, backward_delete_char, delete_char);
+  ok = execute_with_uniarg (n, backward_delete_char, delete_char);
 }
 END_DEFUN
 
@@ -233,15 +240,11 @@ DEFUN ("delete-horizontal-space", delete_horizontal_space)
 Delete all spaces and tabs around point.
 +*/
 {
-  undo_start_sequence ();
-
   while (!eolp () && isspace (following_char ()))
     delete_char ();
 
   while (!bolp () && isspace (preceding_char ()))
     backward_delete_char ();
-
-  undo_end_sequence ();
 }
 END_DEFUN
 
@@ -250,10 +253,8 @@ DEFUN ("just-one-space", just_one_space)
 Delete all spaces and tabs around point, leaving one space.
 +*/
 {
-  undo_start_sequence ();
   FUNCALL (delete_horizontal_space);
   insert_char (' ');
-  undo_end_sequence ();
 }
 END_DEFUN
 
@@ -303,7 +304,7 @@ does nothing.
     target_goalc = 0;
   else
     { /* Find goalc in previous non-blank line. */
-      Marker *m = point_marker ();
+      Marker m = point_marker ();
 
       previous_nonblank_goalc ();
 
@@ -325,7 +326,6 @@ does nothing.
     }
 
   /* Insert indentation.  */
-  undo_start_sequence ();
   if (target_goalc > 0)
     {
       /* If not at EOL on target line, insert spaces & tabs up to
@@ -348,7 +348,6 @@ does nothing.
     }
   else
     ok = bool_to_lisp (insert_tab ());
-  undo_end_sequence ();
 }
 END_DEFUN
 
@@ -356,7 +355,7 @@ static size_t
 previous_line_indent (void)
 {
   size_t cur_indent;
-  Marker *m = point_marker ();
+  Marker m = point_marker ();
 
   FUNCALL (previous_line);
   FUNCALL (beginning_of_line);
@@ -402,10 +401,9 @@ Indentation is done using the `indent-for-tab-command' function.
 
   deactivate_mark ();
 
-  undo_start_sequence ();
   if (insert_newline ())
     {
-      Marker *m = point_marker ();
+      Marker m = point_marker ();
 
       /* Check where last non-blank goalc is. */
       previous_nonblank_goalc ();
@@ -419,6 +417,5 @@ Indentation is done using the `indent-for-tab-command' function.
         FUNCALL (indent_for_tab_command);
       ok = leT;
     }
-  undo_end_sequence ();
 }
 END_DEFUN
