@@ -1,6 +1,6 @@
 /* Registers facility functions
 
-   Copyright (c) 2001, 2003-2006, 2008-2011 Free Software Foundation, Inc.
+   Copyright (c) 2001-2014 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -97,7 +97,7 @@ Puts point before and mark after the inserted text.
       minibuf_clear ();
       reg %= NUM_REGISTERS;
 
-      if (regs[reg].as == NULL)
+      if (regs[reg] == NULL)
         {
           minibuf_error ("Register does not contain text");
           ok = leNIL;
@@ -106,7 +106,7 @@ Puts point before and mark after the inserted text.
         {
           FUNCALL (set_mark_command);
           regnum = reg;
-          execute_with_uniarg (true, uniarg, insert_register, NULL);
+          execute_with_uniarg (uniarg, insert_register, NULL);
           FUNCALL (exchange_point_and_mark);
           deactivate_mark ();
         }
@@ -118,17 +118,17 @@ static void
 write_registers_list (va_list ap _GL_UNUSED_PARAMETER)
 {
   for (size_t i = 0; i < NUM_REGISTERS; ++i)
-    if (regs[i].as != NULL)
+    if (regs[i] != NULL)
       {
-        const char *s = astr_cstr (regs[i].as);
+        const char *s = astr_cstr (estr_get_as (regs[i]));
         while (*s == ' ' || *s == '\t' || *s == '\n')
           s++;
-        size_t len = MIN (20, MAX (0, ((int) get_window_ewidth (cur_wp)) - 6)) + 1;
+        int len = MIN (20, MAX (0, ((int) get_window_ewidth (cur_wp)) - 6)) + 1;
 
-        bprintf ("Register %s contains ", astr_cstr (astr_fmt (isprint (i) ? "%c" : "\\%o", i)));
+        bprintf ("Register %s contains ", astr_cstr (astr_fmt (isprint (i) ? "%c" : "\\%o", (int) i)));
         if (strlen (s) > 0)
           bprintf ("text starting with\n    %.*s\n", len, s);
-        else if (s != astr_cstr (regs[i].as))
+        else if (s != astr_cstr (estr_get_as (regs[i])))
           bprintf ("whitespace\n");
         else
           bprintf ("the empty string\n");
