@@ -1,6 +1,6 @@
 /* Search and replace functions
 
-   Copyright (c) 1997-2012 Free Software Foundation, Inc.
+   Copyright (c) 1997-2014 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -49,7 +49,7 @@ no_upper (const char *s, size_t len, int regex)
 static const char *re_find_err = NULL;
 
 static int
-find_substr (castr as1, castr as2, const char *n, size_t nsize, size_t from, size_t to,
+find_substr (const_astr as, const char *n, size_t nsize,
              bool forward, bool notbol, bool noteol, bool regex, bool icase)
 {
   int ret = -1;
@@ -69,17 +69,10 @@ find_substr (castr as1, castr as2, const char *n, size_t nsize, size_t from, siz
   re_find_err = re_compile_pattern (n, (int) nsize, &pattern);
   pattern.not_bol = notbol;
   pattern.not_eol = noteol;
+  size_t len = astr_len (as);
   if (!re_find_err)
-    /* FIXME: The current implementation memcpys the two strings into
-       a freshly malloced block, i.e. is horribly inefficient for
-       large buffers. */
-    ret = re_search_2 (&pattern,
-                       astr_cstr (as1), (int) astr_len (as1),
-                       astr_cstr (as2), (int) astr_len (as2),
-                       forward ? from : to - 1,
-                       forward ? (to - from) : -(to - 1 - from),
-                       &search_regs,
-                       (int) astr_len (as1) + astr_len (as2));
+    ret = re_search (&pattern, astr_cstr (as), (int) len,
+                     forward ? 0 : len, forward ? len : -len, &search_regs);
 
   if (ret >= 0)
     ret = forward ? search_regs.end[0] : ret;
@@ -88,32 +81,31 @@ find_substr (castr as1, castr as2, const char *n, size_t nsize, size_t from, siz
 }
 
 static bool
-search (size_t o, const char *s, int forward, int regexp)
+search (const char *s, int forward, int regexp)
 {
   size_t ssize = strlen (s);
   if (ssize < 1)
     return false;
 
   /* Attempt match. */
+  size_t o = get_buffer_pt (cur_bp);
   bool notbol = forward ? o > 0 : false;
   bool noteol = forward ? false : o < get_buffer_size (cur_bp);
-  size_t from = forward ? o : 0;
-  size_t to = forward ? get_buffer_size (cur_bp) : o;
-  int pos = find_substr (get_buffer_pre_point (cur_bp), get_buffer_post_point (cur_bp),
-                         s, ssize, from, to, forward, notbol, noteol, regexp,
+  int pos = find_substr (forward ? get_buffer_post_point (cur_bp) : get_buffer_pre_point (cur_bp),
+                         s, ssize, forward, notbol, noteol, regexp,
                          get_variable_bool ("case-fold-search") && no_upper (s, ssize, regexp));
   if (pos < 0)
     return false;
 
-  goto_offset (pos);
+  goto_offset (pos + (forward ? get_buffer_pt (cur_bp) : 0));
   thisflag |= FLAG_NEED_RESYNC;
   return true;
 }
 
-static castr last_search = NULL;
+static const_astr last_search = NULL;
 
 static le *
-do_search (bool forward, bool regexp, castr pattern)
+do_search (bool forward, bool regexp, const_astr pattern)
 {
   le * ok = leNIL;
 
@@ -127,8 +119,8 @@ do_search (bool forward, bool regexp, castr pattern)
     {
       last_search = pattern;
 
-      if (!search (get_buffer_pt (cur_bp), astr_cstr (pattern), forward, regexp))
-        minibuf_error ("Search failed: \"%s\"", pattern);
+      if (!search (astr_cstr (pattern), forward, regexp))
+        minibuf_error ("Search failed: \"%s\"", astr_cstr (pattern));
       else
         ok = leT;
     }
@@ -186,7 +178,7 @@ END_DEFUN
 static le *
 isearch (int forward, int regexp)
 {
-  Marker *old_mark = copy_marker (get_buffer_mark (get_window_bp (cur_wp)));
+  Marker old_mark = copy_marker (get_buffer_mark (get_window_bp (cur_wp)));
 
   set_buffer_isearch (get_window_bp (cur_wp), true);
 
@@ -297,7 +289,10 @@ isearch (int forward, int regexp)
         astr_cat_char (pattern, c);
 
       if (astr_len (pattern) > 0)
-        last = search (cur, astr_cstr (pattern), forward, regexp);
+        {
+          goto_offset (cur);
+          last = search (astr_cstr (pattern), forward, regexp);
+        }
       else
         last = true;
 
@@ -395,20 +390,20 @@ As each match is found, the user must type a character saying
 what to do with it.
 +*/
 {
-  castr find = minibuf_read ("Query replace string: ", "");
+  const_astr find = minibuf_read ("Query replace string: ", "");
   if (find == NULL)
     return FUNCALL (keyboard_quit);
   if (astr_len (find) == 0)
     return leNIL;
   bool find_no_upper = no_upper (astr_cstr (find), astr_len (find), false);
 
-  castr repl = minibuf_read ("Query replace `%s' with: ", "", astr_cstr (find));
+  const_astr repl = minibuf_read ("Query replace `%s' with: ", "", astr_cstr (find));
   if (repl == NULL)
     return FUNCALL (keyboard_quit);
 
   bool noask = false;
   size_t count = 0;
-  while (search (get_buffer_pt (cur_bp), astr_cstr (find), true, false))
+  while (search (astr_cstr (find), true, false))
     {
       int c = ' ';
 
@@ -416,60 +411,57 @@ what to do with it.
         {
           if (thisflag & FLAG_NEED_RESYNC)
             window_resync (cur_wp);
-          for (;;)
-            {
-              minibuf_write
-                ("Query replacing `%s' with `%s' (y, n, !, ., q)? ", astr_cstr (find),
-                 astr_cstr (repl));
-              c = getkey (GETKEY_DEFAULT);
-              if (c == KBD_CANCEL || c == KBD_RET || c == ' ' || c == 'y'
-                  || c == 'n' || c == 'q' || c == '.' || c == '!')
-                break;
-              minibuf_error ("Please answer y, n, !, . or q.");
-              waitkey ();
-            }
+
+          minibuf_write ("Query replacing `%s' with `%s' (y, n, !, ., q)? ", astr_cstr (find),
+                         astr_cstr (repl));
+          c = getkey (GETKEY_DEFAULT);
           minibuf_clear ();
 
           if (c == 'q')			/* Quit immediately. */
             break;
-          else if (c == KBD_CANCEL)	/* C-g */
+          else if (c == KBD_CANCEL)
             {
-             ok = FUNCALL (keyboard_quit);
-             break;
+              ok = FUNCALL (keyboard_quit);
+              break;
             }
-          else if (c == '!')		/* Replace all without asking. */
+          else if (c == '!')
             noask = true;
-          else if (c == 'n' || c == KBD_RET || c == KBD_DEL) /* Do not replace. */
-            continue;
         }
 
-      /* Perform replacement. */
-      ++count;
-      castr case_repl = repl;
-      Region r = region_new (get_buffer_pt (cur_bp) - astr_len (find), get_buffer_pt (cur_bp));
-      if (find_no_upper && get_variable_bool ("case-replace"))
+      if (c == KBD_RET || c == ' ' || c == 'y' || c == 'Y' ||  c == '.' || c == '!')
+        { /* Perform replacement. */
+          ++count;
+          const_astr case_repl = repl;
+          Region r = region_new (get_buffer_pt (cur_bp) - astr_len (find), get_buffer_pt (cur_bp));
+          if (find_no_upper && get_variable_bool ("case-replace"))
+            {
+              int case_type = check_case (estr_get_as (get_buffer_region (cur_bp, r)));
+              if (case_type != 0)
+                case_repl = astr_recase (astr_cpy (astr_new (), repl),
+                                         case_type == 1 ? case_capitalized : case_upper);
+            }
+
+          Marker m = point_marker ();
+          goto_offset (get_region_start (r));
+          replace_estr (astr_len (find), estr_new_astr (case_repl));
+          goto_offset (get_marker_o (m));
+          unchain_marker (m);
+
+          if (c == '.')		/* Replace and quit. */
+            break;
+        }
+      else if (!(c == KBD_RET || c == KBD_DEL || c == 'n' || c == 'N'))
         {
-          int case_type = check_case (get_buffer_region (cur_bp, r).as);
-
-          if (case_type != 0)
-            case_repl = astr_recase (astr_cpy (astr_new (), repl),
-                                     case_type == 1 ? case_capitalized : case_upper);
+          ungetkey (c);
+          ok = false;
+          break;
         }
-
-      Marker *m = point_marker ();
-      goto_offset (r.start);
-      replace_estr (astr_len (find), estr_new_astr (case_repl));
-      goto_offset (get_marker_o (m));
-      unchain_marker (m);
-
-      if (c == '.')		/* Replace and quit. */
-        break;
     }
 
   if (thisflag & FLAG_NEED_RESYNC)
     window_resync (cur_wp);
 
   if (ok)
-    minibuf_write ("Replaced %d occurrences", count);
+    minibuf_write ("Replaced %zu occurrences", count);
 }
 END_DEFUN
