@@ -1,6 +1,6 @@
 /* Undo facility functions
 
-   Copyright (c) 1997-2004, 2008-2011 Free Software Foundation, Inc.
+   Copyright (c) 1997-2014 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -29,8 +29,8 @@
  */
 struct Undo
 {
-  Undo *next;      /* Next undo delta in list. */
-  int type;        /* The type of undo delta. */
+  Undo next;       /* Next undo delta in list. */
+  void *type;      /* The type of undo delta. */
   size_t o;        /* Buffer offset of the undo delta. */
   bool unchanged;  /* Flag indicating that reverting this undo leaves
                       the buffer in an unchanged state. */
@@ -38,32 +38,24 @@ struct Undo
   size_t size;     /* Size of replacement text. */
 };
 
-/* Undo delta types. */
-enum
-{
-  UNDO_REPLACE_BLOCK,		/* Replace a block of characters. */
-  UNDO_START_SEQUENCE,		/* Start a multi operation sequence. */
-  UNDO_END_SEQUENCE		/* End a multi operation sequence. */
-};
-
 /*
  * Save a reverse delta for doing undo.
  */
 static void
-undo_save (int type, size_t o, size_t osize, size_t size)
+undo_save (void *type, size_t o, size_t osize, size_t size)
 {
   if (get_buffer_noundo (cur_bp))
     return;
 
-  Undo * up = (Undo *) XZALLOC (Undo);
-  *up = (Undo) {
+  Undo up = (Undo) XZALLOC (struct Undo);
+  *up = (struct Undo) {
     .next = get_buffer_last_undop (cur_bp),
     .type = type,
   };
 
   up->o = o;
 
-  if (type == UNDO_REPLACE_BLOCK)
+  if (type == undo_save_block)
     {
       up->size = size;
       up->text = get_buffer_region (cur_bp, region_new (o, o + osize));
@@ -76,39 +68,49 @@ undo_save (int type, size_t o, size_t osize, size_t size)
 void
 undo_start_sequence (void)
 {
-  undo_save (UNDO_START_SEQUENCE, get_buffer_pt (cur_bp), 0, 0);
+  if (cur_bp)
+    undo_save (undo_start_sequence, get_buffer_pt (cur_bp), 0, 0);
 }
 
 void
 undo_end_sequence (void)
 {
-  undo_save (UNDO_END_SEQUENCE, 0, 0, 0);
+  if (cur_bp)
+    {
+      Undo up = get_buffer_last_undop (cur_bp);
+      if (up)
+        {
+          if (up->type == undo_start_sequence)
+            set_buffer_last_undop (cur_bp, up->next);
+          else
+            undo_save (undo_end_sequence, 0, 0, 0);
+        }
+
+      /* Update list pointer */
+      if (last_command () != F_undo)
+        set_buffer_next_undop (cur_bp, get_buffer_last_undop (cur_bp));
+    }
 }
 
 void
 undo_save_block (size_t o, size_t osize, size_t size)
 {
-  undo_save (UNDO_REPLACE_BLOCK, o, osize, size);
+  undo_save (undo_save_block, o, osize, size);
 }
 
 /*
  * Revert an action.  Return the next undo entry.
  */
-static Undo *
-revert_action (Undo * up)
+static Undo
+revert_action (Undo up)
 {
-  if (up->type == UNDO_END_SEQUENCE)
-    {
-      undo_start_sequence ();
-      up = up->next;
-      while (up->type != UNDO_START_SEQUENCE)
-        up = revert_action (up);
-      undo_end_sequence ();
-    }
+  if (up->type == undo_end_sequence)
+    for (up = up->next; up->type != undo_start_sequence; up = revert_action (up))
+      ;
 
-  if (up->type != UNDO_END_SEQUENCE)
+  if (up->type != undo_end_sequence)
     goto_offset (up->o);
-  if (up->type == UNDO_REPLACE_BLOCK)
+  if (up->type == undo_save_block)
     replace_estr (up->size, up->text);
   if (up->unchanged)
     set_buffer_modified (cur_bp, false);
@@ -147,7 +149,7 @@ END_DEFUN
  * Set unchanged flags to false.
  */
 void
-undo_set_unchanged (Undo *up)
+undo_set_unchanged (Undo up)
 {
   for (; up; up = up->next)
     up->unchanged = false;
